using System;
using System.Collections.Generic;
using System.Globalization;
using ETHZ.C0284H.Calculator.Exceptions;
using ETHZ.C0284H.Calculator.Instruct.Commands;
using ETHZ.C0284H.Calculator.Instruct.Operators;

namespace ETHZ.C0284H.Calculator.Instruct
{
    /// <summary>
    /// A repository of instructs - a single point that knows, which languages, commands and operators are supported at the moment.
    /// Can parse instructs from the string. Immutable after static construction.
    /// <remarks>Contains 2 types of parse methods: Parse... will throw a <see cref="MissingInstructException"/> on failure, and TryParse...
    /// will return false in case of failure.</remarks>
    /// </summary>
    public static class InstructRepository
    {
        private static readonly Dictionary<string, OperatorInstruct> s_operators = new Dictionary<string, OperatorInstruct>();
        private static readonly Dictionary<string, CommandInstruct> s_commands = new Dictionary<string, CommandInstruct>();
        private static readonly Dictionary<string, LanguageInstruct> s_languages = new Dictionary<string, LanguageInstruct>();

        private static LanguageInstruct s_english;

        /// <summary>
        /// Initializes a repository with known instructs.
        /// </summary>
        static InstructRepository()
        {
            InitCommands();
            InitLanguages();
            InitOperators();
        }

        public static LanguageInstruct English
        {
            get { return s_english; }
        }

        private static void InitOperators()
        {
            var knownOperators = new OperatorInstruct[]{new PlusInstruct(), new MinusInstruct(), new TimesInstruct(), new DivideByInstruct()};

            for (int i = 0; i < knownOperators.Length; i++)
            {
                s_operators.Add(knownOperators[i].InstructCode, knownOperators[i]);
            }
        }

        private static void InitLanguages()
        {
            var knownLanguages = new[]
            {
                s_english = new LanguageInstruct(SupportedLanguage.English,  CultureInfo.CreateSpecificCulture("en-US")), 
                new LanguageInstruct(SupportedLanguage.German, CultureInfo.CreateSpecificCulture("de-CH"))
            };

            for (int i = 0; i < knownLanguages.Length; i++)
            {
                s_languages.Add(knownLanguages[i].InstructText, knownLanguages[i]);
            }
        }

        private static void InitCommands()
        {
            var knownCommands = new CommandInstruct[]
            {
                new ClearInstruct(),
                new EvaluateInstruct(),
                new HelpInstruct(),
                new PrintInstruct(),
                new QuitInstruct()
            };

            for (int i = 0; i < knownCommands.Length; i++)
            {
                s_commands.Add(knownCommands[i].InstructCode, knownCommands[i]);
            }
        }

        public static LanguageInstruct ParseLanguage(string input)
         {
            LanguageInstruct result;

            if (TryParseLanguage(input, out result))
            {
            return result;
            }

            throw new MissingInstructException(input);
         }

        public static bool TryParseLanguage(string input, out LanguageInstruct languageInstruct)
        {
            LanguageInstruct result;

            if (s_languages.TryGetValue(input, out result))
            {
                languageInstruct = result;
                return true;
            }

            languageInstruct = null;
            return false;
        }

         public static CommandInstruct ParseCommand(string input)
         {
             foreach (var command in s_commands.Values)
             {
                 if (input == command.InstructText)
                 {
                     return command;
                 }
             }

             throw new MissingInstructException(input);
         }

         public static bool TryParseCommand(string input, out CommandInstruct command)
         {
             foreach (var candidate in s_commands.Values)
             {
                 if (input == candidate.InstructText)
                 {
                     command = candidate;
                     return true;
                 }
             }

             command = null;
             return false;
         }

        /// <summary>
        /// Sequentially tries to parse input as language, command, operator or operand. The failure, means that there is no such entity in a system.
        /// </summary>
        /// <param name="input">Raw input.</param>
        /// <param name="instruct">Resulting instruct.</param>
        /// <returns>True, in case of success, false -otherwise.</returns>
        public static bool TryParseInstruct(string input, out ICalculatorInstruct instruct)
        {
            LanguageInstruct lang;

            if (TryParseLanguage(input, out lang))
            {
                instruct = lang;
                return true;
            }

            CommandInstruct command;

            if (TryParseCommand(input, out command))
            {
                instruct = command;
                return true;
            }

            OperatorInstruct @operator;

            if (TryParseOperator(input, out @operator))
            {
                instruct = @operator;
                return true;
            }

            double res;
            if (double.TryParse(input, out res))
            {
                instruct = new CalculatorInstructOperand(res);
                return true;
            }

            instruct = null;
            return false;
        }

        public static OperatorInstruct ParseOperator(string input)
        {
            if (input == null)
            {
                throw new ArgumentNullException("input");
            }

            OperatorInstruct result;

            if (TryParseOperator(input, out result))
            {
                return result;
            }
            
            throw new MissingInstructException();
        }

        public static bool TryParseOperator(string input, out OperatorInstruct @operator)
        {
            if (input == null)
            {
                throw new ArgumentNullException("input");
            }

            OperatorInstruct result;

            if (s_operators.TryGetValue(input, out result))
            {
                @operator = result;
                return true;
            }

            @operator = null;
            return false;
        }
    }
}