﻿using System;
using ETHZ.C0284H.Calculator.Exceptions;
using ETHZ.C0284H.Calculator.Instruct.Commands;
using ETHZ.C0284H.Calculator.Instruct.Operators;
using ETHZ.C0284H.Calculator.UI;

namespace ETHZ.C0284H.Calculator.Instruct.Visitor
{
    /// <summary>
    ///     Console executor executes the instructs that a console can handle.
    /// </summary>
    internal class ConsoleExecutor : ICalculatorInstructVisitor
    {
        private readonly UIResourceManager _translator;

        /// <summary>
        /// Creates a new instance of <see cref="ConsoleExecutor"/>.
        /// </summary>
        /// <param name="console">The console.</param>
        /// <param name="calc">The associated calculator.</param>
        public ConsoleExecutor(CalculatorConsole console, ReversePolishNotationCalculator calc)
        {
            if (console == null)
            {
                throw new ArgumentNullException("console");
            }

            if (calc == null)
            {
                throw new ArgumentNullException("console");
            }

            Console = console;
            Calculator = calc;
            _translator = UIResourceManager.Instance;
        }

        protected CalculatorConsole Console { get; set; }

        protected ReversePolishNotationCalculator Calculator { get; set; }



        /// <summary>
        ///     Executor status indicating whether the instruct has been processed.
        /// </summary>
        public bool InstructProcessed { get; protected set; }

        public void VisitCommand(LanguageInstruct command)
        {
            if (command != null)
            {
                SetCurrentLanguage(command);
                PrintCurrentLanguageName();
                PrintHelpMessage();
                InstructProcessed = true;               
            }
        }

        public void VisitCommand(QuitInstruct command)
        {
            Console.Quit();
            InstructProcessed = true;
        }

        public void VisitCommand(HelpInstruct command)
        {
            PrintHelpMessage();
            InstructProcessed = true;
        }

        public void VisitCommand(ClearInstruct command)
        {
            InstructProcessed = false;
        }

        public void VisitCommand(EvaluateInstruct command)
        {
            InstructProcessed = false;
        }

        public void VisitCommand(PrintInstruct command)
        {
            PrintCalculatorStatus();
            InstructProcessed = true;
        }

        public void VisitOperator(DivideByInstruct minusOperator)
        {
            InstructProcessed = false;
        }

        /// <summary>
        ///     Visitor method.
        /// </summary>
        /// <param name="operand"></param>
        public  void VisitOperand(CalculatorInstructOperand operand)
        {
            InstructProcessed = false;
        }

        public void VisitOperator(PlusInstruct plusOperator)
        {
            InstructProcessed = false;
        }

        public void VisitOperator(MinusInstruct minusOperator)
        {
            InstructProcessed = false;
        }

        public void VisitOperator(TimesInstruct minusOperator)
        {
            InstructProcessed = false;
        }

        /// <summary>
        ///     Set current language as specified by the command type.
        /// </summary>
        /// <param name="language">Desired language.</param>
        private void SetCurrentLanguage(LanguageInstruct language)
        {
            try
            {
                _translator.SetLanguage(language);
            }
            catch (MissingResourceException e)
            {
                Console.PrintMessage(e.Message);
            }
        }

        /// <summary>
        ///     Prints the help message.
        /// </summary>
        private void PrintHelpMessage()
        {
            Console.PrintMessage(_translator.GetLocalizedMessage(UIMessageID.MSG_HELP_MESSAGE));
        }

        /// <summary>
        ///     Prints the memory status of calculator.
        /// </summary>
        private void PrintCalculatorStatus()
        {
            Console.PrintMessage(Calculator.ToString());
        }

        /// <summary>
        ///     Prints the name of current language.
        /// </summary>
        private void PrintCurrentLanguageName()
        {
            Console.PrintMessage(string.Format("{0} {1}", _translator.GetLocalizedMessage(UIMessageID.MSG_CURRENT_LANGUAGE), _translator.CurrentLanguageName));
        }
    }
}